@@ -57,14 +57,11 @@ static idmef_analyzer_t *idmef_analyzer;
 
 
 
-static int resolve_failed_fallback(const lml_log_entry_t *log_entry, idmef_node_t *node)
+static int resolve_failed_fallback(idmef_node_t *node, const char *hostname)
 {
         int ret;
-        const char *hostname;
         idmef_address_t *address;
         prelude_string_t *string;
-
-        hostname = lml_log_entry_get_target_hostname(log_entry);
         
         /*
          * we want to know if it's an ip address or an hostname.
@@ -76,17 +73,17 @@ static int resolve_failed_fallback(const lml_log_entry_t *log_entry, idmef_node_
                  */
                 ret = idmef_node_new_name(node, &string);
                 if ( ret < 0 )
-                        return -1;
+                        return ret;
                 
                 prelude_string_set_ref(string, hostname);
         } else {
-                ret = idmef_node_new_address(node, &address, -1);
+                ret = idmef_node_new_address(node, &address, IDMEF_LIST_APPEND);
                 if ( ret < 0 ) 
-                        return -1;
+                        return ret;
 
                 ret = idmef_address_new_address(address, &string);
                 if ( ret < 0 )
-                        return -1;
+                        return ret;
                 
                 prelude_string_set_ref(string, hostname);
         }
@@ -97,7 +94,7 @@ static int resolve_failed_fallback(const lml_log_entry_t *log_entry, idmef_node_
 
 
 
-static int fill_target(idmef_node_t *node, struct addrinfo *ai) 
+static int fill_target_node_from_addrinfo(idmef_node_t *node, struct addrinfo *ai) 
 {
         int ret;
         char str[128];
@@ -106,7 +103,7 @@ static int fill_target(idmef_node_t *node, struct addrinfo *ai)
         prelude_string_t *string;
 
         while ( ai ) {
-                if ( ai->ai_flags & AI_CANONNAME ) {
+                if ( ai->ai_canonname ) {
                         ret = idmef_node_new_name(node, &string);
                         if ( ret < 0 )
                                 return -1;
@@ -115,7 +112,7 @@ static int fill_target(idmef_node_t *node, struct addrinfo *ai)
                                 return -1;
                 }
                 
-                ret = idmef_node_new_address(node, &addr, -1);
+                ret = idmef_node_new_address(node, &addr, IDMEF_LIST_APPEND);
                 if ( ret < 0 )
                         return -1;
 
@@ -146,9 +143,37 @@ static int fill_target(idmef_node_t *node, struct addrinfo *ai)
 }
 
 
-static int fill_analyzer(const lml_log_entry_t *log_entry, idmef_analyzer_t *analyzer)
+
+static int fill_target_node(idmef_node_t *node, const char *host)
 {
         int ret;
+        struct addrinfo *ai, hints;
+        
+        if ( config.no_resolve )
+                ret = resolve_failed_fallback(node, host);
+        else {                
+                memset(&hints, 0, sizeof(hints));
+                hints.ai_flags = AI_CANONNAME;
+                hints.ai_socktype = SOCK_STREAM;
+                
+                ret = getaddrinfo(host, NULL, &hints, &ai);                
+                if ( ret != 0 ) {
+                        prelude_log(PRELUDE_LOG_WARN, "error resolving \"%s\": %s.\n", host, gai_strerror(ret));
+                        return resolve_failed_fallback(node, host);
+                }
+
+                ret = fill_target_node_from_addrinfo(node, ai);
+                freeaddrinfo(ai);
+        }
+
+        return ret;
+}
+
+
+
+static int fill_analyzer(const lml_log_entry_t *log_entry, idmef_analyzer_t *analyzer)
+{
+        int ret = 0;
         const char *tmp;
         idmef_node_t *node;
         prelude_string_t *str;
@@ -173,34 +198,22 @@ static int fill_analyzer(const lml_log_entry_t *log_entry, idmef_analyzer_t *ana
         }
 
         tmp = lml_log_entry_get_target_hostname(log_entry);
-        if ( tmp && ! idmef_analyzer_get_node(analyzer) && ! config.no_resolve ) {
-                struct addrinfo *ai, hints;
-                
+        if ( tmp && ! idmef_analyzer_get_node(analyzer) ) {                
+
                 ret = idmef_analyzer_new_node(analyzer, &node);
                 if ( ret < 0 ) 
                         return -1;
 
-                memset(&hints, 0, sizeof(hints));
-                hints.ai_flags = AI_CANONNAME;
-                hints.ai_socktype = SOCK_STREAM;
-
-                ret = getaddrinfo(tmp, NULL, &hints, &ai);                
-                if ( ret != 0 ) {
-                        prelude_log(PRELUDE_LOG_WARN, "error resolving \"%s\": %s.\n", tmp, gai_strerror(ret));
-                        return resolve_failed_fallback(log_entry, node);
-                }
-
-                fill_target(node, ai);
-                freeaddrinfo(ai);
+                ret = fill_target_node(node, tmp);
         }
 
-        return 0;
+        return ret;
 }
 
 
 static int generate_target(const lml_log_entry_t *log_entry, idmef_alert_t *alert) 
 {
-        int ret;
+        int ret = 0;
         const char *tmp;
         idmef_node_t *node;
         prelude_string_t *str;
@@ -209,7 +222,7 @@ static int generate_target(const lml_log_entry_t *log_entry, idmef_alert_t *aler
 
         target = idmef_alert_get_next_target(alert, NULL);
         if ( ! target ) {
-                ret = idmef_alert_new_target(alert, &target, -1);
+                ret = idmef_alert_new_target(alert, &target, IDMEF_LIST_APPEND);
                 if ( ret < 0 ) 
                         return ret;
         }
@@ -232,28 +245,15 @@ static int generate_target(const lml_log_entry_t *log_entry, idmef_alert_t *aler
         }
 
         tmp = lml_log_entry_get_target_hostname(log_entry);
-        if ( tmp && ! idmef_target_get_node(target) && ! config.no_resolve ) {
-                struct addrinfo *ai, hints;
-                
+        if ( tmp && ! idmef_target_get_node(target) ) {
                 ret = idmef_target_new_node(target, &node);
                 if ( ret < 0 ) 
                         return ret;
 
-                memset(&hints, 0, sizeof(hints));
-                hints.ai_flags = AI_CANONNAME;
-                hints.ai_socktype = SOCK_STREAM;
-
-                ret = getaddrinfo(tmp, NULL, &hints, &ai);                
-                if ( ret != 0 ) {
-                        prelude_log(PRELUDE_LOG_WARN, "error resolving \"%s\": %s.\n", tmp, gai_strerror(ret));
-                        return resolve_failed_fallback(log_entry, node);
-                }
-
-                fill_target(node, ai);
-                freeaddrinfo(ai);
+                ret = fill_target_node(node, tmp);
         }
 
-        return 0;
+        return ret;
 }
 
 
@@ -264,7 +264,7 @@ static int generate_additional_data(idmef_alert_t *alert, const char *meaning, c
         prelude_string_t *str;
         idmef_additional_data_t *adata;
 
-        ret = idmef_alert_new_additional_data(alert, &adata, -1);
+        ret = idmef_alert_new_additional_data(alert, &adata, IDMEF_LIST_APPEND);
         if ( ret < 0 )
                 return ret;
 
@@ -291,16 +291,16 @@ void lml_alert_emit(const lml_log_source_t *ls, const lml_log_entry_t *log, idme
         
         alert = idmef_message_get_alert(message);
         if ( ! alert )
-                goto error;
+                return;
         
         ret = idmef_time_new_from_gettimeofday(&time);
         if ( ret < 0 )
-                goto error;
+                return;
         idmef_alert_set_create_time(alert, time);
         
         ret = idmef_alert_new_detect_time(alert, &time);
         if ( ret < 0 )
-                goto error;
+                return;
 
         idmef_time_set_from_timeval(time, lml_log_entry_get_timeval(log));
         
@@ -308,23 +308,23 @@ void lml_alert_emit(const lml_log_source_t *ls, const lml_log_entry_t *log, idme
         
         if ( lml_log_entry_get_target_hostname(log) || lml_log_entry_get_target_process(log) ) {
                 if ( generate_target(log, alert) < 0 )
-                        goto error;
+                        return;
 
                 if ( cur_analyzer && fill_analyzer(log, cur_analyzer) < 0 )
-                        goto error;
+                        return;
         }
 
         if ( idmef_analyzer )
-                idmef_alert_set_analyzer(alert, idmef_analyzer_ref(idmef_analyzer), 0);
+                idmef_alert_set_analyzer(alert, idmef_analyzer_ref(idmef_analyzer), IDMEF_LIST_PREPEND);
         
         source = lml_log_source_get_name(ls);
         if ( generate_additional_data(alert, "Log received from", source) < 0 )
-                goto error;
+                return;
 
         ptr = lml_log_entry_get_original_log(log);
         if ( ptr ) {
                 if ( generate_additional_data(alert, "Original Log", ptr) < 0 )
-                        goto error;
+                        return;
         }
 
         if ( config.text_output_fd )
@@ -332,9 +332,6 @@ void lml_alert_emit(const lml_log_source_t *ls, const lml_log_entry_t *log, idme
         
         if ( ! config.dry_run ) 
                 prelude_client_send_idmef(config.lml_client, message);
-                
- error:
-        idmef_message_destroy(message);
 }
 
 
