@@ -25,6 +25,7 @@
 #include "libmissing.h"
 
 #include <stdio.h>
+#include <ctype.h>
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -52,9 +53,82 @@
 
 
 lml_config_t config;
+static prelude_bool_t have_file = FALSE;
 static const char *config_file = PRELUDE_LML_CONF;
 
 
+
+static int drop_privilege(void)
+{
+        int ret;
+
+        if ( config.wanted_gid ) {
+                ret = setgid(config.wanted_gid);
+                if ( ret < 0 ) {
+                        prelude_log(PRELUDE_LOG_ERR, "change to GID %d failed: %s.\n",
+                                    (int) config.wanted_gid, strerror(errno));
+                        return ret;
+                }
+                
+                ret = setgroups(1, &config.wanted_gid);
+                if ( ret < 0 ) {
+                        prelude_log(PRELUDE_LOG_ERR, "removal of ancillary groups failed: %s.\n", strerror(errno));
+                        return ret;
+                }
+        }
+
+        
+        if ( config.wanted_uid ) {
+                ret = setuid(config.wanted_uid);
+                if ( ret < 0 ) {
+                        prelude_log(PRELUDE_LOG_ERR, "change to UID %d failed: %s.\n",
+                                    (int) config.wanted_uid, strerror(errno));
+                        return ret;
+                }
+        }
+        
+        return 0;
+}
+
+
+
+
+static int check_file_access(const char *filename)
+{
+        int ret;
+        struct stat st;
+
+        ret = stat(filename, &st);
+        if ( ret < 0 ) {
+                if ( errno == ENOENT )
+                        prelude_log(PRELUDE_LOG_WARN, "* WARNING: %s does not exist.\n", filename);
+                else
+                        prelude_log(PRELUDE_LOG_WARN, "could not stat %s: %s.\n", filename, strerror(errno));
+
+                return -1;
+        }
+
+        if ( ! config.wanted_uid && getuid() == 0 )
+                /* the user started LML as root using --user root */
+                return 0;
+        
+        if ( st.st_uid == config.wanted_uid && st.st_mode & S_IRUSR )
+                return 0;
+
+        if ( st.st_gid == config.wanted_gid && st.st_mode & S_IRGRP )
+                return 0;
+
+        if ( st.st_mode & S_IROTH )
+                return 0;
+
+        prelude_log(PRELUDE_LOG_WARN, "* WARNING: %s is not available for reading to uid %d/gid %d.\n",
+                    filename, config.wanted_uid, config.wanted_gid);
+
+        return -1;
+}
+
+
+
 static int set_conf_file(prelude_option_t *opt, const char *optarg, prelude_string_t *err, void *context)
 {
         config_file = strdup(optarg);
@@ -184,28 +258,16 @@ static int set_pidfile(prelude_option_t *opt, const char *arg, prelude_string_t
         return 0;
 }
 
-
-
 static int set_logfile_prefix_regex(prelude_option_t *opt, const char *arg, prelude_string_t *err, void *context)
-{        
-        if ( config.logfile_prefix_regex )
-                free(config.logfile_prefix_regex);
-        
-        config.logfile_prefix_regex = strdup(arg);
-
-        return 0;
+{
+        return lml_log_format_set_prefix_regex(context, arg);
 }
 
 
 
 static int set_logfile_ts_format(prelude_option_t *opt, const char *arg, prelude_string_t *err, void *context)
-{        
-        if ( config.logfile_ts_format )
-                free(config.logfile_ts_format);
-        
-        config.logfile_ts_format = strdup(arg);
-                
-        return 0;
+{
+        return lml_log_format_set_ts_fmt(context, arg);
 }
 
 
@@ -228,16 +290,11 @@ static int set_text_output(prelude_option_t *opt, const char *arg, prelude_strin
         if ( ret < 0 )
                 return ret;
 
-        if ( strcmp(arg, "stdout") == 0 ) {
+        if ( strcmp(arg, "-") == 0 ) {
                 prelude_io_set_file_io(config.text_output_fd, stdout);
                 return 0;
         }
 
-        else if ( strcmp(arg, "stderr") == 0 ) {
-                prelude_io_set_file_io(config.text_output_fd, stderr);
-                return 0;
-        }
-
         fd = fopen(arg, "w");
         if ( ! fd ) {
                 prelude_log(PRELUDE_LOG_ERR, "could not open %s for writing.\n", arg);
@@ -256,112 +313,80 @@ static int set_file(prelude_option_t *opt, const char *arg, prelude_string_t *er
 {
         int ret;
         lml_log_source_t *ls;
-        regex_list_t *rlist;
+
+        ret = check_file_access(arg);
+        if ( ret < 0 )
+                return 0; /* ignore error */
         
-        ls = lml_log_source_new();
-        if ( ! ls )
+        ret = lml_log_source_new(&ls, context, arg);
+        if ( ret < 0 )
                 return prelude_error_from_errno(errno);
 
-        lml_log_source_set_warning_limit(ls, config.warning_limit);
-        
-        if ( config.logfile_prefix_regex ) {
-                ret = lml_log_source_set_prefix_regex(ls, config.logfile_prefix_regex);
-                if ( ret < 0 )
-                        return ret;
-        }
-
-        if ( config.logfile_ts_format ) {
-                ret = lml_log_source_set_ts_fmt(ls, config.logfile_ts_format);
-                if ( ret < 0 )
-                        return ret;
-        }
-
-        ret = access(arg, R_OK);
-        if ( ret < 0 ) {
-                prelude_string_sprintf(err, "%s does not exist or have wrong permissions", arg);
-                return -1;
-        }
-        
-        ret = lml_log_source_set_name(ls, arg);
-        if ( ret < 0 ) 
-                return ret;
-
-        rlist = regex_init(arg);
-        if ( ! rlist )
-                return -1;
-        
-        ret = file_server_monitor_file(rlist, ls);
-        if ( ret < 0 ) 
-                return ret;
+        if ( ret == 1 ) 
+                return 0;
                 
+        ret = file_server_monitor_file(ls);
+        if ( ret < 0 ) 
+                return ret;
+
+        have_file = TRUE;
+        
         return 0;
 }
 
 
 
-static int destroy_udp_server(prelude_option_t *opt, prelude_string_t *err, void *context)
+static int add_server(lml_log_source_t *ls, const char *addr, unsigned int port)
 {
-        if ( ! config.udp_srvr )
-                return 0;
-                
-        prelude_log(PRELUDE_LOG_INFO, "- closing syslog server listening at %s:%d.\n",
-                    config.udp_srvr_addr, config.udp_srvr_port);
+        config.udp_nserver++;
 
-        free(config.udp_srvr_addr);
-        udp_server_close(config.udp_srvr);
-        config.udp_srvr = NULL;
+        config.udp_server = _prelude_realloc(config.udp_server, sizeof(*config.udp_server) * config.udp_nserver);        
+        if ( ! config.udp_server )
+                return -1;
+        
+        config.udp_server[config.udp_nserver - 1] = udp_server_new(ls, addr, port);        
+        if ( ! config.udp_server[config.udp_nserver - 1] )
+                return -1;
                 
         return 0;
 }
 
 
 
-static int get_udp_server(prelude_option_t *opt, prelude_string_t *out, void *context)
-{
-        if ( ! config.udp_srvr )
-                return 0;
-        
-        return prelude_string_sprintf(out, "%s:%u", config.udp_srvr_addr, config.udp_srvr_port);
-}
-
-
 static int set_udp_server(prelude_option_t *opt, const char *arg, prelude_string_t *err, void *context) 
 {
-        regex_list_t *rlist;
+        int ret;
+        unsigned int port;
+        lml_log_source_t *ls;
+        char *addr, source[512];
         
-        destroy_udp_server(opt, err, context);
-        
-        if ( arg && *arg ) {
-                unsigned int ret, port = 0;
-                
-                ret = prelude_parse_address(arg, &config.udp_srvr_addr, &port);
+        if ( arg && *arg ) {                
+                ret = prelude_parse_address(arg, &addr, &port);
                 if ( ret < 0 )
                         return ret;
-                
-                config.udp_srvr_port = port ? port : DEFAULT_UDP_SERVER_PORT;
-        } 
-        
-        else {
-                config.udp_srvr_addr = strdup("0.0.0.0");
-                config.udp_srvr_port = DEFAULT_UDP_SERVER_PORT;
-        }
-        
-        rlist = regex_init("syslog");
-        if ( ! rlist ) {
-                free(config.udp_srvr_addr);
-                return -1;
-        }
-        
-        config.udp_srvr = udp_server_new(rlist, config.udp_srvr_addr, config.udp_srvr_port);        
-        if ( ! config.udp_srvr ) {
-                free(config.udp_srvr_addr);
-                regex_destroy(rlist);
-                return -1;
+
+                port = port ? port : DEFAULT_UDP_SERVER_PORT;
+        } else {
+                addr = strdup("0.0.0.0");
+                port = DEFAULT_UDP_SERVER_PORT;
         }
 
-        prelude_log(PRELUDE_LOG_INFO, "- Listening for syslog message on %s:%d.\n",
-                    config.udp_srvr_addr, config.udp_srvr_port);
+        snprintf(source, sizeof(source), "%s:%u/udp", addr, port);
+        
+        ret = lml_log_source_new(&ls, context, source);
+        if ( ret < 0 || ret == 1 ) {
+                free(addr);
+                return ret;
+        }
+        
+        ret = add_server(ls, addr, port);
+        free(addr);
+        
+        if ( ret < 0 )
+                return -1;
 
+        prelude_log(PRELUDE_LOG_INFO, "- Listening for syslog message on %s.\n", source);
+        
         return 0;
 }
 
@@ -381,6 +406,72 @@ static int set_warning_limit(prelude_option_t *opt, const char *arg, prelude_str
 }
 
 
+static int set_format(prelude_option_t *opt, const char *arg, prelude_string_t *err, void *context) 
+{
+        lml_log_format_t *format;
+        prelude_option_context_t *octx;
+        
+        format = lml_log_format_new(arg);
+        if ( ! format )
+                return -1;
+        
+        return prelude_option_new_context(opt, &octx, arg, format);
+}
+
+
+
+static int set_user(prelude_option_t *opt, const char *optarg, prelude_string_t *err, void *context)
+{
+        uid_t uid;
+        const char *p;
+        struct passwd *pw;
+
+        for ( p = optarg; isdigit((int) *p); p++ );
+        
+        if ( *p == 0 )
+                uid = atoi(optarg);
+        else {
+                pw = getpwnam(optarg);
+                if ( ! pw ) {
+                        prelude_log(PRELUDE_LOG_ERR, "could not lookup user '%s'.\n", optarg);
+                        return -1;
+                }
+
+                uid = pw->pw_uid;
+        }
+
+        config.wanted_uid = uid;
+        
+        return 0;
+}
+
+
+static int set_group(prelude_option_t *opt, const char *optarg, prelude_string_t *err, void *context)
+{
+        gid_t gid;
+        const char *p;
+        struct group *grp;
+
+        for ( p = optarg; isdigit((int) *p); p++ );
+
+        if ( *p == 0 )
+                gid = atoi(optarg);
+        else {
+                grp = getgrnam(optarg);
+                if ( ! grp ) {
+                        prelude_log(PRELUDE_LOG_ERR, "could not lookup group '%s'.\n", optarg);
+                        return -1;
+                }
+
+                gid = grp->gr_gid;
+        }
+
+        config.wanted_gid = gid;
+        
+        return 0;
+}
+
+
 
 int lml_options_init(prelude_option_t *ropt, int argc, char **argv)
 {
@@ -400,10 +491,19 @@ int lml_options_init(prelude_option_t *ropt, int argc, char **argv)
                            "Print version number", PRELUDE_OPTION_ARGUMENT_NONE,
                            print_version, NULL);
         prelude_option_set_priority(opt, PRELUDE_OPTION_PRIORITY_IMMEDIATE);
-        
-        prelude_option_add(ropt, NULL, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG, 'q', "quiet",
-                           "Quiet mode", PRELUDE_OPTION_ARGUMENT_NONE, set_quiet_mode, NULL);
 
+        prelude_option_add(ropt, NULL, PRELUDE_OPTION_TYPE_CFG|PRELUDE_OPTION_TYPE_CLI, 0, "user",
+                           "Set the user ID used by prelude-lml", PRELUDE_OPTION_ARGUMENT_REQUIRED, set_user, NULL);
+        prelude_option_set_priority(opt, PRELUDE_OPTION_PRIORITY_IMMEDIATE);
+        
+        prelude_option_add(ropt, &opt, PRELUDE_OPTION_TYPE_CFG|PRELUDE_OPTION_TYPE_CLI, 0, "group",
+                           "Set the group ID used by prelude-lml", PRELUDE_OPTION_ARGUMENT_REQUIRED, set_group, NULL);
+        prelude_option_set_priority(opt, PRELUDE_OPTION_PRIORITY_IMMEDIATE);
+        
+        prelude_option_add(ropt, &opt, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG, 'q', "quiet",
+                           "Quiet mode", PRELUDE_OPTION_ARGUMENT_NONE, set_quiet_mode, NULL);
+        prelude_option_set_priority(opt, PRELUDE_OPTION_PRIORITY_IMMEDIATE);
+        
         prelude_option_add(ropt, &opt, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG, 'D', "debug-level",
                            "Debug mode", PRELUDE_OPTION_ARGUMENT_OPTIONAL, set_debug_mode, NULL);
         prelude_option_set_priority(opt, PRELUDE_OPTION_PRIORITY_IMMEDIATE);
@@ -413,6 +513,11 @@ int lml_options_init(prelude_option_t *ropt, int argc, char **argv)
                            set_daemon_mode, NULL);
         prelude_option_set_priority(opt, PRELUDE_OPTION_PRIORITY_FIRST);
         
+        prelude_option_add(ropt, &opt, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG, 'P', "pidfile",
+                           "Write Prelude LML PID to specified file",
+                           PRELUDE_OPTION_ARGUMENT_REQUIRED, set_pidfile, NULL);
+        prelude_option_set_priority(opt, PRELUDE_OPTION_PRIORITY_IMMEDIATE);
+
         prelude_option_add(ropt, NULL, PRELUDE_OPTION_TYPE_CLI, 0, "text-output",
                            "Dump alert to stdout, or to the specified file", PRELUDE_OPTION_ARGUMENT_REQUIRED,
                            set_text_output, NULL);
@@ -425,21 +530,7 @@ int lml_options_init(prelude_option_t *ropt, int argc, char **argv)
                            "Configuration file to use", PRELUDE_OPTION_ARGUMENT_REQUIRED,
                            set_conf_file, NULL);
         prelude_option_set_priority(opt, PRELUDE_OPTION_PRIORITY_IMMEDIATE);
-        
-        prelude_option_add(ropt, &opt, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG, 'P', "pidfile",
-                           "Write Prelude LML PID to specified pidfile",
-                           PRELUDE_OPTION_ARGUMENT_REQUIRED, set_pidfile, NULL);
-        
-        prelude_option_set_priority(opt, PRELUDE_OPTION_PRIORITY_IMMEDIATE);
-        
-        prelude_option_add(ropt, &opt, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG|
-                           PRELUDE_OPTION_TYPE_WIDE, 's', "udp-srvr",
-                           "address:port pair to listen to syslog to UDP messages (default port 514)", 
-                           PRELUDE_OPTION_ARGUMENT_OPTIONAL, set_udp_server, get_udp_server);
-
-        prelude_option_set_destroy_callback(opt, destroy_udp_server);        
-        prelude_option_set_priority(opt, PRELUDE_OPTION_PRIORITY_LAST);
-                
+                                
         prelude_option_add(ropt, NULL, all_hook, 0, "max-rotation-time-offset",
                            "Specifies the maximum time difference, in seconds, between the time " \
                            "of logfiles rotation. If this amount is reached, a high "   \
@@ -469,34 +560,62 @@ int lml_options_init(prelude_option_t *ropt, int argc, char **argv)
         prelude_option_add(ropt, NULL, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG, 0, "no-resolve",
                            "Do not attempt to resolve target address (useful for profiling)",
                            PRELUDE_OPTION_ARGUMENT_NONE, set_no_resolve, NULL);
+
+        prelude_option_add(ropt, &opt, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG,
+                           0, "format", NULL, PRELUDE_OPTION_ARGUMENT_REQUIRED, set_format, NULL);
+        prelude_option_set_priority(opt, PRELUDE_OPTION_PRIORITY_LAST);
         
-        prelude_option_add(ropt, NULL, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG,
+        prelude_option_add(opt, NULL, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG,
                            't', "time-format", "Specify the input timestamp format", PRELUDE_OPTION_ARGUMENT_REQUIRED,
                            set_logfile_ts_format, NULL);
         
-        prelude_option_add(ropt, NULL, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG,
+        prelude_option_add(opt, NULL, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG,
                            'p', "prefix-regex", "Specify the input prefix format", PRELUDE_OPTION_ARGUMENT_REQUIRED,
                            set_logfile_prefix_regex, NULL);
         
-        prelude_option_add(ropt, &opt, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG,
+        prelude_option_add(opt, NULL, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG,
                            'f', "file", "Specify a file to monitor (use \"-\" for standard input)",
                            PRELUDE_OPTION_ARGUMENT_REQUIRED, set_file, NULL);
         
+        prelude_option_add(opt, NULL, PRELUDE_OPTION_TYPE_CLI|PRELUDE_OPTION_TYPE_CFG, 's', "udp-server",
+                           "address:port pair to listen to syslog to UDP messages (default port 514)", 
+                           PRELUDE_OPTION_ARGUMENT_OPTIONAL, set_udp_server, NULL);
+        
         ret = prelude_option_read(ropt, &config_file, &argc, argv, &err, NULL);
         if ( ret < 0 ) {
+                if ( prelude_error_get_code(ret) == PRELUDE_ERROR_EOF )
+                        return -1;
+                
                 if ( err )
                         prelude_log(PRELUDE_LOG_WARN, "%s.\n", prelude_string_get_string(err));
                 else
-                        prelude_perror(ret, "failed parsing LML options");
+                        prelude_perror(ret, "error processing options");
                 
                 return -1;
         }
+
+        while ( ret < argc )
+                prelude_log(PRELUDE_LOG_WARN, "Unhandled command line argument: '%s'.\n", argv[ret++]);
         
-        if ( config.batch_mode && config.udp_srvr ) {
+        if ( config.batch_mode && config.udp_nserver ) {
                 prelude_log(PRELUDE_LOG_WARN, "UDP server and batch modes can't be used together.\n");
                 return -1;
         }
 
+        if ( config.ignore_metadata && ! config.batch_mode ) {
+                prelude_log(PRELUDE_LOG_WARN, "--ignore-metadata is only supported in batch mode.\n");
+                return -1;
+        }
+
+        if ( ! have_file && ! config.udp_nserver ) {
+                prelude_log(PRELUDE_LOG_WARN, "No file or UDP server available for monitoring: terminating.\n");
+                return -1;
+        }
+        
+        ret = drop_privilege();
+        if ( ret < 0 )
+                return -1;
+
         if ( config.dry_run )
                 return 0;
         
@@ -521,6 +640,6 @@ int lml_options_init(prelude_option_t *ropt, int argc, char **argv)
 
                 return -1;
         }
-                
+        
         return 0;
 }
